import { createContext, useContext, useEffect, useMemo, useState } from "react";

type LangStruct<Languages, key> = Record<
  key & string,
  Record<Languages & string, string>
>;

export const LanguageContext = createContext("en");

type PropagationType = React.Dispatch<React.SetStateAction<string>>;

export const LanguagePropagation = createContext<
  Record<string, PropagationType>
>({});

export function useLanguage<Languages, Keys>(
  languageStructure: LangStruct<Languages, Keys>
) {
  const langContext = useContext(LanguageContext) as Languages & string;
  const [, set] = useState<string>("");
  const keyID = useMemo(
    () =>
      Math.random()
        .toString(36)
        .substring(2, 8 + 2),
    []
  );
  const propagation = useContext(LanguagePropagation);

  useEffect(() => {
    propagation[keyID] = set;
    return () => {
      delete propagation[keyID];
    };
  }, []);

  return useMemo(() => {
    return Object.assign(
      {},
      ...(
        Object.keys(languageStructure) as Array<
          keyof Record<Keys & string, string>
        >
      ).map((e) => {
        return {
          [e]: languageStructure[e][langContext],
        };
      })
    ) as Record<Keys & string, string>;
  }, [langContext]) as Record<Keys & string, string>;
}

export function LanguageProvider<Languages>({
  children,
  lang,
  default_lang,
}: {
  children: any;
  lang: Languages & string;
  default_lang: Languages & string;
}) {
  const [inited, setInited] = useState(false);
  const propagation = useContext(LanguagePropagation);

  useEffect(() => {
    if (!inited) return;
    const hooks = Object.values(propagation) as PropagationType[];
    for (let i = 0; i < hooks.length; i++) hooks[i](lang);
  }, [lang]);

  useEffect(() => {
    setInited(true);
  }, []);
  return (
    <LanguageContext.Provider value={inited ? (lang as string) : default_lang}>
      {children}
    </LanguageContext.Provider>
  );
}

type languagesList =
  | "af"
  | "agq"
  | "ak"
  | "am"
  | "ar"
  | "as"
  | "asa"
  | "ast"
  | "az"
  | "bas"
  | "be"
  | "bem"
  | "bez"
  | "bg"
  | "bm"
  | "bn"
  | "bo"
  | "br"
  | "brx"
  | "bs"
  | "ca"
  | "ccp"
  | "ce"
  | "cgg"
  | "chr"
  | "ckb"
  | "cs"
  | "cy"
  | "da"
  | "dav"
  | "de"
  | "dje"
  | "dsb"
  | "dua"
  | "dyo"
  | "dz"
  | "ebu"
  | "ee"
  | "el"
  | "en"
  | "eo"
  | "es"
  | "et"
  | "eu"
  | "ewo"
  | "fa"
  | "ff"
  | "fi"
  | "fil"
  | "fo"
  | "fr"
  | "fur"
  | "fy"
  | "ga"
  | "gd"
  | "gl"
  | "gsw"
  | "gu"
  | "guz"
  | "gv"
  | "ha"
  | "haw"
  | "he"
  | "hi"
  | "hr"
  | "hsb"
  | "hu"
  | "hy"
  | "id"
  | "ig"
  | "ii"
  | "is"
  | "it"
  | "ja"
  | "jgo"
  | "jmc"
  | "ka"
  | "kab"
  | "kam"
  | "kde"
  | "kea"
  | "khq"
  | "ki"
  | "kk"
  | "kkj"
  | "kl"
  | "kln"
  | "km"
  | "kn"
  | "ko"
  | "kok"
  | "ks"
  | "ksb"
  | "ksf"
  | "ksh"
  | "kw"
  | "ky"
  | "lag"
  | "lb"
  | "lg"
  | "lkt"
  | "ln"
  | "lo"
  | "lrc"
  | "lt"
  | "lu"
  | "luo"
  | "luy"
  | "lv"
  | "mas"
  | "mer"
  | "mfe"
  | "mg"
  | "mgh"
  | "mgo"
  | "mk"
  | "ml"
  | "mn"
  | "mr"
  | "ms"
  | "mt"
  | "mua"
  | "my"
  | "mzn"
  | "naq"
  | "nb"
  | "nd"
  | "nds"
  | "ne"
  | "nl"
  | "nmg"
  | "nn"
  | "nnh"
  | "nus"
  | "nyn"
  | "om"
  | "or"
  | "os"
  | "pa"
  | "pl"
  | "ps"
  | "pt"
  | "qu"
  | "rm"
  | "rn"
  | "ro"
  | "rof"
  | "ru"
  | "rw"
  | "rwk"
  | "sah"
  | "saq"
  | "sbp"
  | "se"
  | "seh"
  | "ses"
  | "sg"
  | "shi"
  | "si"
  | "sk"
  | "sl"
  | "smn"
  | "sn"
  | "so"
  | "sq"
  | "sr"
  | "sv"
  | "sw"
  | "ta"
  | "te"
  | "teo"
  | "tg"
  | "th"
  | "ti"
  | "to"
  | "tr"
  | "tt"
  | "twq"
  | "tzm"
  | "ug"
  | "uk"
  | "ur"
  | "uz"
  | "vai"
  | "vi"
  | "vun"
  | "wae"
  | "wo"
  | "xog"
  | "yav"
  | "yi"
  | "yo"
  | "yue"
  | "zgh"
  | "zh"
  | "zu";

export function useSystemLanguage(languages: Array<languagesList>) {
  const [current, set] = useState<languagesList>("en");

  useEffect(() => {
    const lang = navigator.language.split("-")[0] as languagesList;
    if (!languages.includes(lang)) return;
    set(lang);
  }, []);

  return current;
}

export type LangKeys<LangStruct> = keyof LangStruct;
